#include <stdlib.h>
#include <stdio.h>
#include <omp.h>
#include <math.h>

#ifdef _OPENACC
#include <accel.h>
#endif



//return ||y|| = ||alpha * y + A * x||
float mat_vec(float * restrict A, float * restrict x, 
                    int n, float alpha, float * restrict y)
{
   int i,j;
   float norm;

#pragma acc kernels present(A[0:n*n])
{
   for(i = 0; i < n; ++i){
      float tmp = 0.0;
      for(j = 0; j < n; ++j)
         tmp += A[i*n+j] * x[j];
      y[i] += tmp;
   }

   //compute ||y||^2
   norm= 0.0;
   for(i = 0; i < n; ++i)
      norm += y[i] * y[i];
}

   return sqrt(norm);
}



float mat_vec_host(float *A, float *x, int n, float alpha, float *y);

int main(int argc, char* argv[])
{

   if(argc<3){printf("Usage: %s <number of rows/cols> <number of mat_vec products>\n",argv[0]); exit(-1); }

   //warmup GPU
#ifdef _OPENACC
   acc_init(acc_device_nvidia);
#endif

   //allocate data
   int n = atoi(argv[1]);
   int m = atoi(argv[2]);
   float* A;
   A = (float*) malloc(sizeof(float) * n * n);
   float* x = (float*) malloc(sizeof(float) * n);
   float* y_h = (float*) malloc(sizeof(float) * n);
   float* y = (float*) malloc(sizeof(float) * n);

   //initialize A and x, y
   int i,j;
   for(i = 0; i < n; ++i){
      x[i] = rand()/(float)RAND_MAX;
      y[i] = rand()/(float)RAND_MAX;
      y_h[i] = y[i];
      for(j = 0; j < n; ++j)
         A[i * n + j] = rand()/(float)RAND_MAX;
   }
   float norm_acc; 
   double start, end;


#pragma acc data copy_in(A[0:n*n])
{
   norm_acc = mat_vec(A, x, n, 1.0 , y);
   start = omp_get_wtime();
   for(i=0;i < m; ++i)
      norm_acc = mat_vec(A, x, n, 1.0/norm_acc, y);
   end = omp_get_wtime();
}






   /*****************************
    * VERIFY RESUTLS
    *****************************/
   float norm = mat_vec_host(A, x, n, 1.0, y_h);
   for(i=0;i < m; ++i)
      norm = mat_vec_host(A, x, n, 1.0/norm, y_h);

   if(fabs(norm-norm_acc)/fabs(norm) < 0.001)
      printf("Runtime: %f sec.\nRESULTS OKAY!\n",end-start);
   else
      printf("Runtime: %f sec. \nRESULT CHECK FAILED: %.16f\n", end-start, fabs(norm-norm_acc)/fabs(norm));

   return 0;
}


//return ||y|| = ||alpha * y + A * x||
float mat_vec_host(float *A, float *x, int n, float alpha, float *y)
{
   int i,j;
   float norm;

   for(i = 0; i < n; ++i){
      float tmp = 0.0;
      for(j = 0; j < n; ++j)
         tmp += A[i*n+j] * x[j];
      y[i] += tmp;
   }

   //compute ||y||
   norm= 0.0;
   for(i = 0; i < n; ++i)
      norm += y[i] * y[i];

   return sqrt(norm);
}

