/**************************
 * module load cuda
 * module load pgi
 *************************/

#include <stdlib.h>
#include <stdio.h>
#include <omp.h>
#include <math.h>

#ifdef _OPENACC
#include <accel.h>
#endif



//return ||y|| = ||alpha * y + beta * x||
float vec_vec(float beta , float * x, 
           float alpha, float * y, int n)
{
   int i;
   float norm;

#pragma acc kernels
{
   for(i = 0; i < n; ++i)
      y[i] = alpha * y[i] + beta * x[i];

   //compute ||y||^2
   norm = 0.0;
   for(i = 0; i < n; ++i)
      norm += y[i] * y[i];
}

   return sqrt(norm);
}



float vec_vec_host(float beta, float *x, float alpha, float *y, int n);

int main(int argc, char* argv[])
{

   //warmup GPU
#ifdef _OPENACC
   acc_init(acc_device_nvidia);
#endif

   if(argc<3){printf("Usage: %s <size of vector> <number of vec_vec products>\n",argv[0]); exit(-1); }

   //allocate data
   int n = atoi(argv[1]);
   int m = atoi(argv[2]);
   float* x = (float*) malloc(sizeof(float) * n);
   float* y_h = (float*) malloc(sizeof(float) * n);
   float* y = (float*) malloc(sizeof(float) * n);
   float alpha = 2.1;
   float beta = 1.3;

   //initialize A and x, y
   int i,j;
   for(i = 0; i < n; ++i){
      x[i] = rand()/(float)RAND_MAX;
      y[i] = rand()/(float)RAND_MAX;
      y_h[i] = y[i];
   }

   float norm_acc = vec_vec(beta, x, alpha, y, n);
   double start = omp_get_wtime();
   for(i=0;i < m; ++i)
      norm_acc = vec_vec(beta, x, 1.0/norm_acc, y, n);
   double end = omp_get_wtime();






   /*****************************
    * VERIFY RESUTLS
    *****************************/
   float norm = vec_vec_host(beta, x, alpha, y_h, n);
   for(i=0;i < m; ++i)
      norm= vec_vec_host(beta, x, 1.0/norm, y, n);
   if(fabs(norm-norm_acc)/fabs(norm) < 1e-3)
      printf("Runtime: %f sec.\nRESULTS OKAY!\n",end-start);
   else
      printf("Runtime: %f sec. RESULT CHECK FAILED: %f\n", end-start, fabs(norm-norm_acc)/fabs(norm));

   return 0;
}


float vec_vec_host(float beta, float *x, float alpha, float *y, int n)
{
   int i;
   float norm;

   for(i = 0; i < n; ++i)
      y[i] = alpha * y[i] + beta * x[i];

   //compute ||y||^2
   norm = 0.0;
   for(i = 0; i < n; ++i)
      norm += y[i] * y[i];

   return sqrt(norm);
}

